' Fig. 8.4: CTime2.vb
' Represents time and contains overloaded constructors.

Class CTime2
   Inherits Object

   ' declare Integers for hour, minute and second
   Private mHour As Integer   ' 0 - 23
   Private mMinute As Integer ' 0 - 59
   Private mSecond As Integer ' 0 - 59

   ' constructor initializes each variable to zero and
   ' ensures that each CTime2 object starts in consistent state
   Public Sub New()
      SetTime()
   End Sub ' New

   ' CTime2 constructor: hour supplied;
   ' minute and second default to 0
   Public Sub New(ByVal hourValue As Integer)
      SetTime(hourValue)
   End Sub ' New

   ' CTime2 constructor: hour and minute supplied; 
   ' second defaulted to 0
   Public Sub New(ByVal hourValue As Integer, _
      ByVal minuteValue As Integer)

      SetTime(hourValue, minuteValue)
   End Sub ' New

   ' CTime2 constructor: hour, minute and second supplied
   Public Sub New(ByVal hourValue As Integer, _
      ByVal minuteValue As Integer, ByVal secondValue As Integer)

      SetTime(hourValue, minuteValue, secondValue)
   End Sub ' New

   ' CTime2 constructor: another CTime2 object supplied
   Public Sub New(ByVal timeValue As CTime2)
      SetTime(timeValue.mHour, timeValue.mMinute, timeValue.mSecond)
   End Sub ' New

   ' set new time value using universal time;
   ' perform validity checks on data;
   ' set invalid values to zero
   Public Sub SetTime(Optional ByVal hourValue As Integer = 0, _
      Optional ByVal minuteValue As Integer = 0, _
      Optional ByVal secondValue As Integer = 0)

      ' perform validity checks on hour, then set hour
      If (hourValue >= 0 AndAlso hourValue < 24) Then
         mHour = hourValue
      Else
         mHour = 0
      End If

      ' perform validity checks on minute, then set minute
      If (minuteValue >= 0 AndAlso minuteValue < 60) Then
         mMinute = minuteValue
      Else
         mMinute = 0
      End If

      ' perform validity checks on second, then set second
      If (secondValue >= 0 AndAlso secondValue < 60) Then
         mSecond = secondValue
      Else
         mSecond = 0
      End If

   End Sub ' SetTime

   ' convert String to universal-time format
   Public Function ToUniversalString() As String
      Return String.Format("{0}:{1:D2}:{2:D2}", _
         mHour, mMinute, mSecond)
   End Function ' ToUniversalString

   ' convert to String in standard-time format
   Public Function ToStandardString() As String
      Dim suffix As String = " PM"
      Dim format As String = "{0}:{1:D2}:{2:D2}"
      Dim standardHour As Integer

      ' determine whether time is AM or PM
      If mHour < 12 Then
         suffix = " AM"
      End If

      ' convert from universal-time format to standard-time format
      If (mHour = 12 OrElse mHour = 0) Then
         standardHour = 12
      Else
         standardHour = mHour Mod 12
      End If

      Return String.Format(format, standardHour, mMinute, _
         mSecond) & suffix
   End Function ' ToStandardString

End Class ' CTime2

' *************************************************************
' * (C) Copyright 2002 by Deitel & Associates, Inc.           *
' *     and Prentice Hall.                                    *
' * All Rights Reserved.                                      *
' *                                                           *
' * DISCLAIMER: The authors and publisher of this book have   *
' * used their best efforts in preparing the book. These      *
' * efforts include the development, research, and testing of *
' * the theories and programs to determine their              *
' * effectiveness. The authors and publisher make no warranty *
' * of any kind, expressed or implied, with regard to these   *
' * programs or to the documentation contained in these books.*
' * The authors and publisher shall not be liable in any event*
' * for incidental or consequential damages in connection     *
' * with, or arising out of, the furnishing, performance, or  *
' * use of these programs.                                    *
' *************************************************************